@@ -56,6 +56,7 @@ typedef unsigned long long u64;
 
 u64 cliserv_magic = 0x00420281861253LL;
 u64 opts_magic = 0x49484156454F5054LL;
+u64 rep_magic = 0x3e889045565a9LL;
 #define INIT_PASSWD "NBDMAGIC"
 
 #define INFO(a) do { } while(0)
@@ -148,4 +149,20 @@ u64 ntohll(u64 a) {
 					 * served */
 
 /* Options that the client can select to the server */
-#define NBD_OPT_EXPORT_NAME	(1 << 0)	/* Client wants to select a named export (is followed by length and name of export) */
+#define NBD_OPT_EXPORT_NAME	(1)	/** Client wants to select a named export (is followed by name of export) */
+#define NBD_OPT_ABORT		(2)	/** Client wishes to abort negotiation */
+#define NBD_OPT_LIST		(3)	/** Client request list of supported exports (not followed by data) */
+
+/* Replies the server can send during negotiation */
+#define NBD_REP_ACK		(1)	/** ACK a request. Data: option number to be acked */
+#define NBD_REP_SERVER		(2)	/** Reply to NBD_OPT_LIST (one of these per server; must be followed by NBD_REP_ACK to signal the end of the list */
+#define NBD_REP_FLAG_ERROR	(1 << 31)	/** If the high bit is set, the reply is an error */
+#define NBD_REP_ERR_UNSUP	(1 | NBD_REP_FLAG_ERROR)	/** Client requested an option not understood by this version of the server */
+#define NBD_REP_ERR_POLICY	(2 | NBD_REP_FLAG_ERROR)	/** Client requested an option not allowed by server configuration. (e.g., the option was disabled) */
+#define NBD_REP_ERR_INVALID	(3 | NBD_REP_FLAG_ERROR)	/** Client issued an invalid request */
+#define NBD_REP_ERR_PLATFORM	(4 | NBD_REP_FLAG_ERROR)	/** Option not supported on this platform */
+
+/* Global flags */
+#define NBD_FLAG_FIXED_NEWSTYLE (1 << 0)	/* new-style export that actually supports extending */
+/* Flags from client to server. Only one such option currently. */
+#define NBD_FLAG_C_FIXED_NEWSTYLE NBD_FLAG_FIXED_NEWSTYLE
